/*
* ps 内部不知道咋调整亮度的，我直接加上一个颜色dt跟PS还是有偏差，目前H,S可以跟ps配对。
*/
#ifdef GL_ES
precision mediump float;
#endif

varying vec2 v_texCoord;

uniform float uH;			//[-180~+180]
uniform float uS;			//[-100~+100]
uniform float uL;


vec3 RGBToHSL(vec3 col)
{
       float red   = col.r;
    float green = col.g;
    float blue  = col.b;

    float minc  = min(min(col.r, col.g), col.b);
    float maxc  = max(max(col.r, col.g), col.b);
    float delta = maxc - minc;

    float lum = (minc + maxc) * 0.5;
    float sat = 0.0;
    float hue = 0.0;

    if (lum > 0.0 && lum < 1.0) {
        float mul = (lum < 0.5)  ?  (lum)  :  (1.0-lum);
        sat = delta / (mul * 2.0);
    }

    vec3 masks = vec3(
        (maxc == red   && maxc != green) ? 1.0 : 0.0,
        (maxc == green && maxc != blue)  ? 1.0 : 0.0,
        (maxc == blue  && maxc != red)   ? 1.0 : 0.0
    );

    vec3 adds = vec3(
              ((green - blue ) / delta),
        2.0 + ((blue  - red  ) / delta),
        4.0 + ((red   - green) / delta)
    );

    float deltaGtz = (delta > 0.0) ? 1.0 : 0.0;

    hue += dot( adds, masks );
    hue *= deltaGtz;
    hue /= 6.0;

    if (hue < 0.0)
        hue += 1.0;

    return vec3( hue, sat, lum);
}
                                                                                                                                                                                                    
float HueToRGB(float f1, float f2, float hue)
{
        if (hue < 0.0)
            hue += 1.0;
        else if (hue > 1.0)
            hue -= 1.0;
        float res;
        if ((6.0 * hue) < 1.0)
            res = f1 + (f2 - f1) * 6.0 * hue;
        else if ((2.0 * hue) < 1.0)
            res = f2;
        else if ((3.0 * hue) < 2.0)
            res = f1 + (f2 - f1) * ((2.0 / 3.0) - hue) * 6.0;
        else
            res = f1;
        return res;
}
                                                                                                                                                                                                    
vec3 HSLToRGB(vec3 col)
{
        const float onethird = 1.0 / 3.0;
    const float twothird = 2.0 / 3.0;
    const float rcpsixth = 6.0;

    float hue = col.x;
    float sat = col.y;
    float lum = col.z;

    vec3 xt = vec3(
        rcpsixth * (hue - twothird),
        0.0,
        rcpsixth * (1.0 - hue)
    );

    if (hue < twothird) {
        xt.r = 0.0;
        xt.g = rcpsixth * (twothird - hue);
        xt.b = rcpsixth * (hue      - onethird);
    } 

    if (hue < onethird) {
        xt.r = rcpsixth * (onethird - hue);
        xt.g = rcpsixth * hue;
        xt.b = 0.0;
    }

    xt = min( xt, 1.0 );

    float sat2   =  2.0 * sat;
    float satinv =  1.0 - sat;
    float luminv =  1.0 - lum;
    float lum2m1 = (2.0 * lum) - 1.0;
    vec3  ct     = (sat2 * xt) + satinv;

    vec3 rgb;
    if (lum >= 0.5)
         rgb = (luminv * ct) + lum2m1;
    else rgb =  lum    * ct;

    return rgb;
}


void main() {
    
    float flipy = 1.0 - v_texCoord.y;
    
    vec4 texColor=texture2D(CC_Texture0, vec2(v_texCoord.x,flipy));
	
	

		
	if(texColor.a != 0.0){
    float r=texColor.r;
    float g=texColor.g;
    float b=texColor.b;
    float a=texColor.a;
    //convert rgb to hsl
    float h=0.0;
    float s=0.0;
    float l=0.0;
   
	vec3 hslresult=RGBToHSL(vec3(r,g,b));

	h=hslresult.x;
	s=hslresult.y;
	l=hslresult.z;

    //(h,s,l)+(dH,dS,dL) -> (h,s,l)
     h=h+uH/360.0;
	
     s=min(1.0,max(0.0,s*(   (100.0 + uS) / 100.0 )));
     l=l;
 
    vec4 finalColor;
    
	if (h < 0.0) 
		h += 1.0; 
	else if (h > 1.0) 
		h -= 1.0; 
		

	finalColor=vec4(HSLToRGB(vec3(h,s,l)).xyz,a);
 
    finalColor=clamp(finalColor+vec4(uL,uL,uL,0.0),
	0.0,1.0);
    
    gl_FragColor=finalColor;
    }
	else
		discard;
}

